@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: alloc.c,v 1.12.4.2 2010/10/21 04:07:53 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: alloc.c,v 1.17 2011/09/04 11:08:33 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - alloc.c */
@@ -48,19 +48,6 @@ static char *RCSid() { return RCSid("$Id: alloc.c,v 1.12.4.2 2010/10/21 04:07:53
 #endif
 #include "util.h"
 
-#if defined(MSDOS) && defined(__TURBOC__) && !defined(DOSX286)
-# include <alloc.h>		/* for farmalloc, farrealloc */
-#endif
-
-#if defined(_Windows) && !defined(WIN32)
-# include <windows.h>
-# include <windowsx.h>
-# define farmalloc(s) GlobalAllocPtr(GHND,s)
-# define farrealloc(p,s) GlobalReAllocPtr(p,s,GHND)
-#endif
-
-
-
 #ifndef GP_FARMALLOC
 # ifdef FARALLOC
 #  define GP_FARMALLOC(size) farmalloc ((size))
@@ -336,7 +323,7 @@ void
 gpfree(generic *p)
 {
 #ifdef _Windows
-    HGLOBAL hGlobal = GlobalHandle(SELECTOROF(p));
+    HGLOBAL hGlobal = GlobalHandle(p);
     GlobalUnlock(hGlobal);
     GlobalFree(hGlobal);
 #else
