@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: color.c,v 1.85.2.3 2009/11/04 16:10:48 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: color.c,v 1.98.2.3 2013/03/10 18:23:59 broeker Exp $"); }
 #endif
 
 /* GNUPLOT - color.c */
@@ -107,6 +107,7 @@ make_palette()
 
     /* ask for suitable number of colours in the palette */
     i = term->make_palette(NULL);
+    sm_palette.colors = i;
     if (i == 0) {
 	/* terminal with its own mapping (PostScript, for instance)
 	   It will not change palette passed below, but non-NULL has to be
@@ -122,9 +123,12 @@ make_palette()
     }
 
     /* set the number of colours to be used (allocated) */
-    sm_palette.colors = i;
-    if (sm_palette.use_maxcolors > 0 && i > sm_palette.use_maxcolors)
-	sm_palette.colors = sm_palette.use_maxcolors;
+    if (sm_palette.use_maxcolors > 0) {
+	if (sm_palette.colorMode == SMPAL_COLOR_MODE_GRADIENT)
+	    sm_palette.colors = i;	/* EAM Sep 2010 - could this be a constant? */
+	else if (i > sm_palette.use_maxcolors)
+	    sm_palette.colors = sm_palette.use_maxcolors;
+    }
 
     if (prev_palette.colorFormulae < 0
 	|| sm_palette.colorFormulae != prev_palette.colorFormulae
@@ -136,7 +140,8 @@ make_palette()
 	|| sm_palette.colors != prev_palette.colors) {
 	/* print the message only if colors have changed */
 	if (interactive)
-	fprintf(stderr, "smooth palette in %s: available %i color positions; using %i of them\n", term->name, i, sm_palette.colors);
+	    fprintf(stderr, "smooth palette in %s: using %i of %i available color positions\n",
+	    		term->name, sm_palette.colors, i);
     }
 
     prev_palette = sm_palette;
@@ -183,6 +188,7 @@ set_color(double gray)
 	return;
     color.type = TC_FRAC;
     color.value = gray;
+    color.lt = 0;
     term->set_color(&color);
 }
 
@@ -194,6 +200,7 @@ set_rgbcolor(int rgblt)
 	return;
     color.type = TC_RGB;
     color.lt = rgblt;
+    color.value = 0;
     term->set_color(&color);
 }
 
@@ -206,16 +213,11 @@ void ifilled_quadrangle(gpiPoint* icorners)
     term->filled_polygon(4, icorners);
 
     if (pm3d.hidden3d_tag) {
-
 	int i;
 
-	/* Colour has changed, so we must apply line properties again.
-	 * FIXME: It would be cleaner to apply the general line properties
-	 * outside this loop, and limit ourselves to apply_pm3dcolor().
-	 */
-	static struct lp_style_type lp = DEFAULT_LP_STYLE_TYPE;
-	lp_use_properties(&lp, pm3d.hidden3d_tag);
-	term_apply_lp_properties(&lp);
+	/* It should be sufficient to set only the color, but for some */
+	/* reason this causes the svg terminal to lose the fill type.  */
+	term_apply_lp_properties(&pm3d_border_lp);
 
 	term->move(icorners[0].x, icorners[0].y);
 	for (i = 3; i >= 0; i--) {
@@ -353,17 +355,18 @@ draw_inside_color_smooth_box_postscript(FILE * out)
 
 
 
-/* plot the colour smooth box for from terminal's integer coordinates
+/* plot a colour smooth box bounded by the terminal's integer coordinates
    [x_from,y_from] to [x_to,y_to].
-   This routine is for non-postscript files, as it does explicitly the loop
+   This routine is for non-postscript files, as it does an explicit loop
    over all thin rectangles
  */
 static void
 draw_inside_color_smooth_box_bitmap(FILE * out)
 {
     int steps = 128; /* I think that nobody can distinguish more colours drawn in the palette */
-    int i, xy, xy2, xy_from, xy_to;
-    double xy_step, gray;
+    int i, j, xy, xy2, xy_from, xy_to;
+    int jmin = 0;
+    double xy_step, gray, range;
     gpiPoint corners[4];
 
     (void) out;			/* to avoid "unused parameter" warning */
@@ -372,33 +375,58 @@ draw_inside_color_smooth_box_bitmap(FILE * out)
 	corners[1].x = corners[2].x = color_box.bounds.xright;
 	xy_from = color_box.bounds.ybot;
 	xy_to = color_box.bounds.ytop;
+	xy_step = (color_box.bounds.ytop - color_box.bounds.ybot) / (double)steps;
     } else {
 	corners[0].y = corners[1].y = color_box.bounds.ybot;
 	corners[2].y = corners[3].y = color_box.bounds.ytop;
 	xy_from = color_box.bounds.xleft;
 	xy_to = color_box.bounds.xright;
+	xy_step = (color_box.bounds.xright - color_box.bounds.xleft) / (double)steps;
     }
-    xy_step = (color_box.rotation == 'h' ? color_box.bounds.xright - color_box.bounds.xleft : color_box.bounds.ytop - color_box.bounds.ybot) / (double) steps;
+    range = (xy_to - xy_from);
 
-    for (i = 0; i < steps; i++) {
-	gray = (double) i / steps;	/* colours equidistantly from [0,1] */
+    for (i = 0, xy2 = xy_from; i < steps; i++) {
+
+	/* Start from one pixel beyond the previous box */
+	xy = xy2;
+	xy2 = xy_from + (int) (xy_step * (i + 1));
+
+	/* Set the colour for the next range increment */
+	/* FIXME - The "1 +" seems wrong, yet it improves the placement in gd */
+	gray = (double)(1 + xy - xy_from) / range;
 	if (sm_palette.positive == SMPAL_NEGATIVE)
 	    gray = 1 - gray;
-	/* Set the colour (also for terminals which support extended specs). */
 	set_color(gray);
-	xy = xy_from + (int) (xy_step * i);
-	xy2 = xy_from + (int) (xy_step * (i + 1));
+
+	/* If this is a defined palette, make sure that the range increment */
+	/* does not straddle a palette segment boundary. If it does, split  */
+	/* it into two parts.                                               */
+	if (sm_palette.colorMode == SMPAL_COLOR_MODE_GRADIENT)
+	    for (j=jmin; j<sm_palette.gradient_num; j++) {
+		int boundary = xy_from + (int)(sm_palette.gradient[j].pos * range);
+		if (xy >= boundary) {
+		    jmin = j;
+		} else {
+		    if (xy2 > boundary) {
+			xy2 = boundary;
+			i--;
+			break;
+		    }
+		}
+		if (xy2 < boundary)
+		    break;
+	    }
+
 	if (color_box.rotation == 'v') {
 	    corners[0].y = corners[1].y = xy;
-	    corners[2].y = corners[3].y = (i == steps - 1) ? xy_to : xy2;
+	    corners[2].y = corners[3].y = GPMIN(xy_to,xy2+1);
 	} else {
 	    corners[0].x = corners[3].x = xy;
-	    corners[1].x = corners[2].x = (i == steps - 1) ? xy_to : xy2;
+	    corners[1].x = corners[2].x = GPMIN(xy_to,xy2+1);
 	}
 #ifdef EXTENDED_COLOR_SPECS
-	if (supply_extended_color_specs) {
+	if (supply_extended_color_specs)
 	    corners[0].spec.gray = -1;	/* force solid color */
-	}
 #endif
 	/* print the rectangle with the given colour */
 	if (default_fillstyle.fillstyle == FS_EMPTY)
@@ -455,8 +483,22 @@ cbtick_callback(
 
     /* draw label */
     if (text) {
-	/* get offset */
+	int just;
 	int offsetx, offsety;
+
+	/* Skip label if we've already written a user-specified one here */
+#	define MINIMUM_SEPARATION 0.001
+	while (userlabels) {
+	    if (fabs((place - userlabels->position) / (CB_AXIS.max - CB_AXIS.min))
+		<= MINIMUM_SEPARATION) {
+		text = NULL;
+		break;
+	    }
+	    userlabels = userlabels->next;
+	}
+#	undef MINIMUM_SEPARATION
+
+	/* get offset */
 	map3d_position_r(&(axis_array[axis].ticdef.offset),
 			 &offsetx, &offsety, "cbtics");
 	/* User-specified different color for the tics text */
@@ -471,16 +513,22 @@ cbtick_callback(
 		    hrotate = axis_array[axis].tic_rotate;
 	    if (len > 0) y3 -= len; /* add outer tics len */
 	    if (y3<0) y3 = 0;
+	    just = hrotate ? LEFT : CENTRE;
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;
 	    write_multiline(x2+offsetx, y3+offsety, text,
-			    (hrotate ? LEFT : CENTRE), CENTRE, hrotate,
+			    just, JUST_CENTRE, hrotate,
 			    axis_array[axis].ticdef.font);
 	    if (hrotate)
 		(*term->text_angle)(0);
 	} else {
 	    unsigned int x3 = color_box.bounds.xright + (term->h_char);
 	    if (len > 0) x3 += len; /* add outer tics len */
+	    just = LEFT;
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;	    
 	    write_multiline(x3+offsetx, y2+offsety, text,
-			    LEFT, CENTRE, 0.0,
+			    just, JUST_CENTRE, 0.0,
 			    axis_array[axis].ticdef.font);
 	}
 	term_apply_lp_properties(&border_lp);	/* border linetype */
@@ -600,7 +648,7 @@ draw_color_smooth_box(int plot_mode)
     /* Optimized version of the smooth colour box in postscript. Advantage:
        only few lines of code is written into the output file.
      */
-    if (gppsfile)
+    if ((term->flags & TERM_IS_POSTSCRIPT) != 0)
 	draw_inside_color_smooth_box_postscript(out);
     else
 	draw_inside_color_smooth_box_bitmap(out);
