@@ -1,5 +1,5 @@
 /*
- * $Id: gp_types.h,v 1.50 2011/04/16 04:55:26 sfeam Exp $
+ * $Id: gp_types.h,v 1.50.2.2 2014/01/31 05:22:06 sfeam Exp $
  */
 
 /* GNUPLOT - gp_types.h */
@@ -116,10 +116,12 @@ typedef enum PLOT_SMOOTH {
     SMOOTH_CSPLINES,
     SMOOTH_SBEZIER,
     SMOOTH_UNIQUE,
+    SMOOTH_UNWRAP,
     SMOOTH_FREQUENCY,
     SMOOTH_CUMULATIVE,
     SMOOTH_KDENSITY,
-    SMOOTH_CUMULATIVE_NORMALISED
+    SMOOTH_CUMULATIVE_NORMALISED,
+    SMOOTH_MONOTONE_CSPLINE
 } PLOT_SMOOTH;
 
 /* FIXME HBB 20000521: 'struct value' and its part, 'cmplx', should go
