@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.82.2.3 2012/07/17 19:09:52 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.82.2.5 2013/10/27 22:13:43 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - hidden3d.c */
@@ -1099,6 +1099,7 @@ build_networks(struct surface_points *plots, int pcount)
 	 * plot styles are mapped to others, that are genuinely
 	 * available in 3d. */
 	switch (this_plot->plot_style) {
+	case PM3DSURFACE:
 	case LINESPOINTS:
 	case STEPS:
 	case FSTEPS:
@@ -1173,9 +1174,17 @@ build_networks(struct surface_points *plots, int pcount)
 
 	/* This is a special flag indicating that the user specified an	*/
 	/* explicit surface color in the splot command.			*/
+	/* FIXME:  Currently never set.					*/
 	if (above == LT_SINGLECOLOR-1)
 	    above = below = LT_SINGLECOLOR;
 
+	/* We will not actually draw PM3D surfaces here, but their 	*/
+	/* edges can be used to calculate occlusion of lines, including */
+	/* the plot borders. (NB: the PM3D surface will _not_ be hidden */
+	/* by other non-PM3D surfaces.					*/
+	if (this_plot->plot_style == PM3DSURFACE)
+	    above = below = LT_NODRAW;
+
 	/* calculate the point symbol type: */
 	/* Assumes that upstream functions have made sure this is
 	 * initialized sensibly --- thou hast been warned */
@@ -1241,6 +1250,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    }
 
 		    switch (this_plot->plot_style) {
+		    case PM3DSURFACE:
 		    case LINESPOINTS:
 		    case STEPS:
 		    case FSTEPS:
@@ -1326,6 +1336,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    = -3;
 
 		switch (this_plot->plot_style) {
+		case PM3DSURFACE:
 		case LINESPOINTS:
 		case STEPS:
 		case FSTEPS:
