@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: misc.c,v 1.138.2.16 2013/10/27 22:13:43 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: misc.c,v 1.138.2.19 2014/09/10 04:15:54 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - misc.c */
@@ -835,6 +835,9 @@ lp_parse(struct lp_style_type *lp, TBOOLEAN allow_ls, TBOOLEAN allow_point)
 		} else if (equals(c_token,"bgnd")) {
 		    *lp = background_lp;
 		    c_token++;
+		} else if (equals(c_token,"nodraw")) {
+		    lp->l_type = LT_NODRAW;
+		    c_token++;
 		} else {
 		    /* These replace the base style */
 		    new_lt = int_expression();
@@ -863,7 +866,7 @@ lp_parse(struct lp_style_type *lp, TBOOLEAN allow_ls, TBOOLEAN allow_point)
 		if (set_pal++)
 		    break;
 		c_token++;
-		if (almost_equals(c_token, "rgb$color")) {
+		if (almost_equals(c_token, "rgb$color") || isstring(c_token)) {
 		    c_token--;
 		    parse_colorspec(&(newlp.pm3d_color), TC_RGB);
 		} else if (almost_equals(c_token, "pal$ette")) {
@@ -1154,6 +1157,12 @@ parse_colorspec(struct t_colorspec *tc, int options)
     } else if (options >= TC_VARIABLE && almost_equals(c_token,"var$iable")) {
 	tc->type = TC_VARIABLE;
 	c_token++;
+
+    /* New: allow to skip the rgb keyword, as in  'plot $foo lc "blue"' */
+    } else if (isstring(c_token)) {
+	tc->type = TC_RGB;
+	tc->lt = parse_color_name();
+
     } else {
 	int_error(c_token, "colorspec option not recognized");
     }
@@ -1342,7 +1351,7 @@ arrow_parse(
 void
 get_image_options(t_image *image)
 {
-    if (equals(c_token, "failsafe")) {
+    if (almost_equals(c_token, "pix$els") || equals(c_token, "failsafe")) {
 	c_token++;
 	image->fallback = TRUE;
     }
