@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.133.2.5 2013/11/10 18:44:22 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.133.2.10 2014/09/14 08:32:44 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -457,34 +457,21 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 static char *
 xDateTimeFormat(double x, char *b, int mode)
 {
-# ifndef SEC_OFFS_SYS
-#  define SEC_OFFS_SYS 946684800
-# endif
-    time_t xtime_position = SEC_OFFS_SYS + x;
-    struct tm *pxtime_position = gmtime(&xtime_position);
+    struct tm tm;
+
     switch (mode) {
     case MOUSE_COORDINATES_XDATE:
-	sprintf(b, "%d. %d. %04d", pxtime_position->tm_mday, (pxtime_position->tm_mon) + 1,
-# if 1
-		(pxtime_position->tm_year) + ((pxtime_position->tm_year <= 68) ? 2000 : 1900)
-# else
-		((pxtime_position->tm_year) < 100) ? (pxtime_position->tm_year) : (pxtime_position->tm_year) - 100
-		/*              (pxtime_position->tm_year)+1900 */
-# endif
-	    );
+	ggmtime(&tm, x);
+	sprintf(b, "%d. %d. %04d", tm.tm_mday, tm.tm_mon + 1, tm.tm_year);
 	break;
     case MOUSE_COORDINATES_XTIME:
-	sprintf(b, "%d:%02d", pxtime_position->tm_hour, pxtime_position->tm_min);
+	ggmtime(&tm, x);
+	sprintf(b, "%d:%02d", tm.tm_hour, tm.tm_min);
 	break;
     case MOUSE_COORDINATES_XDATETIME:
-	sprintf(b, "%d. %d. %04d %d:%02d", pxtime_position->tm_mday, (pxtime_position->tm_mon) + 1,
-# if 1
-		(pxtime_position->tm_year) + ((pxtime_position->tm_year <= 68) ? 2000 : 1900),
-# else
-		((pxtime_position->tm_year) < 100) ? (pxtime_position->tm_year) : (pxtime_position->tm_year) - 100,
-		/*              (pxtime_position->tm_year)+1900, */
-# endif
-		pxtime_position->tm_hour, pxtime_position->tm_min);
+	ggmtime(&tm, x);
+	sprintf(b, "%d. %d. %04d %d:%02d", tm.tm_mday, tm.tm_mon + 1, tm.tm_year,
+		tm.tm_hour, tm.tm_min);
 	break;
     case MOUSE_COORDINATES_TIMEFMT:
 	/* FIXME HBB 20000507: timefmt is for *reading* timedata, not
@@ -1621,7 +1608,13 @@ event_buttonpress(struct gp_event_t *ge)
     } else if (ALMOST2D) {
         if (!setting_zoom_region) {
 	    if (1 == b) {
-	      /* not bound in 2d graphs */
+	      /* "pause button1" or "pause any" takes precedence over key bindings */
+		if (paused_for_mouse & PAUSE_BUTTON1) {
+		    load_mouse_variables(mouse_x, mouse_y, TRUE, b);
+		    trap_release = TRUE;	/* Don't trigger on release also */
+		    return;
+		}
+
 	    } else if (2 == b) {
 		/* not bound in 2d graphs */
 	    } else if (3 == b && 
@@ -2005,7 +1998,9 @@ event_reset(struct gp_event_t *ge)
 	/* This hack is necessary on some systems in order to prevent one  */
 	/* character of input from being swallowed when the plot window is */
 	/* closed. But which systems, exactly?                             */
-	if (term && (!strncmp("x11",term->name,3) || !strncmp("wxt",term->name,3)))
+	if (term && (!strncmp("x11",term->name,3) 
+		 || !strncmp("wxt",term->name,3)
+		 || !strncmp("qt",term->name,2)))
 	    ungetc('\n',stdin);
     }
 
