@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.66.2.4 2013/12/22 18:50:13 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.66.2.7 2014/07/12 23:58:43 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -1021,7 +1021,7 @@ is_builtin_function(int t_num)
 t_iterator *
 check_for_iteration()
 {
-    char *errormsg = "Expecting iterator \tfor [<var> = <start> : <end>]\n\t\t\tor\tfor [<var> in \"string of words\"]";
+    char *errormsg = "Expecting iterator \tfor [<var> = <start> : <end> {: <incr>}]\n\t\t\tor\tfor [<var> in \"string of words\"]";
     int nesting_depth = 0;
     t_iterator *iter = NULL;
     t_iterator *this_iter = NULL;
@@ -1037,6 +1037,7 @@ check_for_iteration()
 	int iteration_current;
 	int iteration = 0;
 	TBOOLEAN empty_iteration;
+	TBOOLEAN just_once = FALSE;
 
 	c_token++;
 	if (!equals(c_token++, "[") || !isletter(c_token))
@@ -1052,6 +1053,8 @@ check_for_iteration()
 	    if (equals(c_token,":")) {
 	    	c_token++;
 	    	iteration_increment = int_expression();
+		if (iteration_increment == 0)
+		    int_error(c_token-1, errormsg);
 	    }
 	    if (!equals(c_token++, "]"))
 	    	int_error(c_token-1, errormsg);
@@ -1077,16 +1080,26 @@ check_for_iteration()
 	    int_error(c_token-1, errormsg);
 
 	iteration_current = iteration_start;
-	
-	empty_iteration = iteration_udv 
-	    && ((iteration_end - iteration_start) * iteration_increment < 0);
-        
-	/* allocating a node of the linked list and initializing its fields */
-	/* iterating just once is the same as not iterating at all, 
-	 * so we skip building the node in that case */
-	if (iteration_increment 
-	&& (iteration_start != iteration_end)
-	&& (abs(iteration_end - iteration_start) >= abs(iteration_increment))) {
+
+	empty_iteration = FALSE;	
+	if ( (iteration_udv != NULL)
+	&&   ((iteration_end > iteration_start && iteration_increment < 0)
+	   || (iteration_end < iteration_start && iteration_increment > 0))) {
+		empty_iteration = TRUE;
+		FPRINTF((stderr,"Empty iteration\n"));
+	}
+
+	/* Allocating a node of the linked list nested iterations. */
+	/* Iterating just once is the same as not iterating at all */
+	/* so we skip building the node in that case.		   */
+	if (iteration_start == iteration_end)
+	    just_once = TRUE;
+	if (iteration_start < iteration_end && iteration_end < iteration_start + iteration_increment)
+	    just_once = TRUE;
+	if (iteration_start > iteration_end && iteration_end > iteration_start + iteration_increment)
+	    just_once = TRUE;
+
+	if (!just_once) {
 	    this_iter = gp_alloc(sizeof(t_iterator), "iteration linked list");
 	    this_iter->iteration_udv = iteration_udv; 
 	    this_iter->iteration_string = iteration_string;
@@ -1097,7 +1110,7 @@ check_for_iteration()
 	    this_iter->iteration = iteration;
 	    this_iter->done = FALSE;
 	    this_iter->really_done = FALSE;
-	    this_iter->empty_iteration = FALSE;
+	    this_iter->empty_iteration = empty_iteration;
 	    this_iter->next = NULL;
 	    this_iter->prev = NULL;
 	    if (nesting_depth == 0) {
@@ -1111,7 +1124,7 @@ check_for_iteration()
 	    }
 	    iter->prev = this_iter; /* a shortcut: making the list circular */
 
-	    /* if one iteration in the chain is empty, the whole chain of iterations is empty, too */
+	    /* if one iteration in the chain is empty, the subchain of nested iterations is too */
 	    if (!iter->empty_iteration) 
 		iter->empty_iteration = empty_iteration;
 
@@ -1169,13 +1182,17 @@ next_iteration(t_iterator *iter)
     } else
 	this_iter->iteration_udv->udv_value.v.int_val = this_iter->iteration_current;
     
-    /* no need to check for increment <> 0 here, 
-     * because that case was already caught in check_for_iteration */
-    this_iter->done = 
-	!((this_iter->iteration_end - this_iter->iteration_current - this_iter->iteration_increment)
-	   * this_iter->iteration_increment >= 0);
+    /* Mar 2014 revised to avoid integer overflow */
+    if (this_iter->iteration_increment > 0
+    &&  this_iter->iteration_end - this_iter->iteration_current < this_iter->iteration_increment)
+	this_iter->done = TRUE;
+    else if (this_iter->iteration_increment < 0
+    &&  this_iter->iteration_end - this_iter->iteration_current > this_iter->iteration_increment)
+	this_iter->done = TRUE;
+    else
+	this_iter->done = FALSE;
     
-    /* We return false only if we're, erm, really done */
+    /* We return false only if we're, um, really done */
     this_iter = iter;
     while (this_iter) {
 	condition = condition || (!this_iter->done);
