@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.18 2013/10/23 18:31:53 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.20 2014/06/15 10:33:33 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - plot.c */
@@ -690,19 +690,32 @@ RECOVER_FROM_ERROR_IN_DASH:
 	    }
     }
 
+    /* take commands from stdin */
+    if (noinputfiles)
+	while (!com_line())
+	    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
+
 #ifdef _Windows
-    /* On Windows 'persist' is handled by keeping the main input loop running. */
+    /* On Windows, handle 'persist' by keeping the main input loop running (windows/wxt), */
+    /* but only if there are any windows open. Note that qt handles this properly. */
     if (persist_cl) {
-	interactive = TRUE;
-	while (!com_line());
-	interactive = FALSE;
-    } else
+	if (WinAnyWindowOpen()) {
+#ifdef WGP_CONSOLE
+	    if (!interactive) {
+		/* no further input from pipe */
+		while (WinAnyWindowOpen())
+		win_sleep(100);
+	    } else
 #endif
-    {
-	/* take commands from stdin */
-	if (noinputfiles)
-	    while (!com_line());
+	    {
+		interactive = TRUE;
+		while (!com_line())
+		    ctrlc_flag = FALSE; /* reset asynchronous Ctrl-C flag */
+		interactive = FALSE;
+	    }
+	}
     }
+#endif
 
 #if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && defined(GNUPLOT_HISTORY)
 #if !defined(HAVE_ATEXIT) && !defined(HAVE_ON_EXIT)
