@@ -1,5 +1,5 @@
 /*
- * $Id: tables.h,v 1.68.2.2 2010/06/26 06:43:47 sfeam Exp $
+ * $Id: tables.h,v 1.80 2011/10/08 00:07:41 sfeam Exp $
  */
 
 /* GNUPLOT - tables.h */
@@ -80,7 +80,7 @@ enum set_id {
     S_FONTPATH, S_FORMAT,
     S_GRID, S_HIDDEN3D, S_HISTORYSIZE, S_ISOSAMPLES, S_KEY,
     S_KEYTITLE, S_LABEL,
-    S_LINESTYLE, S_LOADPATH, S_LOCALE, S_LOGSCALE, S_MACROS,
+    S_LINESTYLE, S_LINETYPE, S_LOADPATH, S_LOCALE, S_LOGSCALE, S_MACROS,
     S_MAPPING, S_MARGIN, S_LMARGIN, S_RMARGIN, S_TMARGIN, S_BMARGIN, S_MISSING,
 #ifdef USE_MOUSE
     S_MOUSE,
@@ -91,9 +91,9 @@ enum set_id {
     S_OFFSETS, S_ORIGIN, SET_OUTPUT, S_PARAMETRIC,
     S_PALETTE, S_PM3D, S_COLORBOX, S_COLORNAMES,
     S_CBLABEL, S_CBRANGE, S_CBTICS, S_NOCBTICS, S_MCBTICS, S_NOMCBTICS,
-    S_CBDATA, S_CBDTICS, S_NOCBDTICS, S_CBMTICS, S_NOCBMTICS,
-    S_OBJECT, S_PLOT, S_POINTSIZE, S_POLAR, S_PRINT,
-    S_RRANGE, S_SAMPLES, S_SIZE, S_SURFACE, S_STYLE, 
+    S_CBDATA, S_CBDTICS, S_NOCBDTICS, S_CBMTICS, S_NOCBMTICS, S_OBJECT,
+    S_PLOT, S_POINTINTERVALBOX, S_POINTSIZE, S_POLAR, S_PRINT, S_PSDIR,
+    S_SAMPLES, S_SIZE, S_SURFACE, S_STYLE, 
     S_TABLE, S_TERMINAL, S_TERMOPTIONS,
     S_TICS, S_TICSCALE, S_TICSLEVEL, S_TIMEFMT, S_TIMESTAMP, S_TITLE,
     S_TRANGE, S_URANGE, S_VARIABLES, S_VERSION, S_VIEW, S_VRANGE,
@@ -111,6 +111,8 @@ enum set_id {
     S_ZDATA, S_ZDTICS, S_NOZDTICS, S_ZLABEL, S_ZMTICS, S_NOZMTICS, S_ZRANGE,
     S_ZTICS, S_NOZTICS,
 
+    S_RTICS, S_NORTICS, S_RRANGE, S_RAXIS,
+
     S_ZERO, S_ZEROAXIS, S_XZEROAXIS, S_X2ZEROAXIS, S_YZEROAXIS, S_Y2ZEROAXIS,
     S_ZZEROAXIS
 };
@@ -136,7 +138,9 @@ enum set_key_id {
     S_KEY_HEIGHT, S_KEY_TITLE, S_KEY_NOTITLE,
     S_KEY_FONT, S_KEY_TEXTCOLOR,
     S_KEY_AUTOTITLES, S_KEY_NOAUTOTITLES,
-    S_KEY_DEFAULT, S_KEY_ON, S_KEY_OFF
+    S_KEY_DEFAULT, S_KEY_ON, S_KEY_OFF,
+    S_KEY_MAXCOLS, S_KEY_MAXROWS,
+    S_KEY_FRONT, S_KEY_NOFRONT
 };
 
 enum set_colorbox_id {
@@ -154,7 +158,7 @@ enum set_palette_id {
     S_PALETTE_GRAY, S_PALETTE_COLOR, S_PALETTE_RGBFORMULAE,
     S_PALETTE_NOPS_ALLCF, S_PALETTE_PS_ALLCF, S_PALETTE_MAXCOLORS,
     S_PALETTE_DEFINED, S_PALETTE_FILE, S_PALETTE_FUNCTIONS,
-    S_PALETTE_MODEL, S_PALETTE_GAMMA
+    S_PALETTE_MODEL, S_PALETTE_GAMMA, S_PALETTE_CUBEHELIX
 };
 
 enum set_pm3d_id {
@@ -182,8 +186,8 @@ enum show_style_id {
     SHOW_STYLE_INVALID,
     SHOW_STYLE_DATA, SHOW_STYLE_FUNCTION, SHOW_STYLE_LINE,
     SHOW_STYLE_FILLING, SHOW_STYLE_ARROW, 
-    SHOW_STYLE_CIRCLE, SHOW_STYLE_RECTANGLE,
-    SHOW_STYLE_INCREMENT, SHOW_STYLE_HISTOGRAM
+    SHOW_STYLE_CIRCLE, SHOW_STYLE_ELLIPSE, SHOW_STYLE_RECTANGLE,
+    SHOW_STYLE_INCREMENT, SHOW_STYLE_HISTOGRAM, SHOW_STYLE_BOXPLOT
 };
 
 enum filledcurves_opts_id {
@@ -192,6 +196,7 @@ enum filledcurves_opts_id {
     /* requirement: FILLEDCURVES_ATX1 = FILLEDCURVES_X1+4 */
     FILLEDCURVES_ATX1, FILLEDCURVES_ATY1, FILLEDCURVES_ATX2, FILLEDCURVES_ATY2,
     FILLEDCURVES_ATXY,
+    FILLEDCURVES_ATR,
     FILLEDCURVES_ABOVE, FILLEDCURVES_BELOW,
     FILLEDCURVES_BETWEEN
 };
