@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.7 2013/06/23 22:13:35 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.8 2013/12/12 20:49:57 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util.c */
@@ -608,6 +608,14 @@ gprintf(
 	    snprintf(dest, remaining_space, temp, x);
 	    break;
 	    /*}}} */
+	    /*{{{  Forward compatibility: treat %h as %g */
+	case 'h':
+	case 'H':
+	    t[0] = 'g';
+	    t[1] = 0;
+	    snprintf(dest, remaining_space, temp, x);
+	    break;
+	    /*}}} */
 	    /*{{{  l --- mantissa to current log base */
 	case 'l':
 	    {
