@@ -1,5 +1,5 @@
 /*
- * $Id: util.h,v 1.36 2011/11/10 05:15:58 sfeam Exp $
+ * $Id: util.h,v 1.36.2.3 2013/06/23 22:13:36 markisch Exp $
  */
 
 /* GNUPLOT - util.h */
@@ -118,10 +118,15 @@ void graph_error __PROTO(());
 void squash_spaces __PROTO((char *));
 
 TBOOLEAN existdir __PROTO((const char *));
+TBOOLEAN existfile(const char *);
 
 char *getusername __PROTO((void));
 
 TBOOLEAN contains8bit __PROTO((const char *s));
 TBOOLEAN utf8toulong __PROTO((unsigned long * wch, const char ** str));
+size_t strlen_utf8 __PROTO((const char *s));
+size_t gp_strlen __PROTO((const char *s));
+char * gp_strchrn __PROTO((const char *s, int N));
+TBOOLEAN streq __PROTO((const char *a, const char *b));
 
 #endif /* GNUPLOT_UTIL_H */
