@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util3d.c,v 1.27.2.4 2009/01/07 22:58:27 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util3d.c,v 1.44.2.1 2012/01/11 23:42:23 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util3d.c */
@@ -174,19 +174,19 @@ mat_mult(
 void
 edge3d_intersect(
     struct coordinate GPHUGE *points,	/* the points array */
-    int i,				/* line segment from point i-1 to point i */
+    int i0, int i1,
     double *ex, double *ey, double *ez)	/* the point where it crosses an edge */
 {
     int count;
-    double ix = points[i - 1].x;
-    double iy = points[i - 1].y;
-    double iz = points[i - 1].z;
-    double ox = points[i].x;
-    double oy = points[i].y;
-    double oz = points[i].z;
+    double ix = points[i0].x;
+    double iy = points[i0].y;
+    double iz = points[i0].z;
+    double ox = points[i1].x;
+    double oy = points[i1].y;
+    double oz = points[i1].z;
     double x, y, z;		/* possible intersection point */
 
-    if (points[i].type == INRANGE) {
+    if (points[i0].type == INRANGE) {
 	/* swap points around so that ix/ix/iz are INRANGE and ox/oy/oz are OUTRANGE */
 	x = ix;
 	ix = ox;
@@ -431,17 +431,17 @@ edge3d_intersect(
 TBOOLEAN			/* any intersection? */
 two_edge3d_intersect(
     struct coordinate GPHUGE *points,	/* the points array */
-    int i,				/* line segment from point i-1 to point i */
+    int i0, int i1,
     double *lx, double *ly, double *lz)	/* lx[2], ly[2], lz[2]: points where it crosses edges */
 {
     int count;
     /* global axis_array[FIRST_{X,Y,Z}_AXIS].{min,max} */
-    double ix = points[i - 1].x;
-    double iy = points[i - 1].y;
-    double iz = points[i - 1].z;
-    double ox = points[i].x;
-    double oy = points[i].y;
-    double oz = points[i].z;
+    double ix = points[i0].x;
+    double iy = points[i0].y;
+    double iz = points[i0].z;
+    double ox = points[i1].x;
+    double oy = points[i1].y;
+    double oz = points[i1].z;
     double t[6];
     double swap;
     double x, y, z;		/* possible intersection point */
@@ -900,119 +900,21 @@ map3d_xyz(
     out->z = Res[2] / Res[3];
     /* store z for later color calculation */
     out->real_z = z;
-#ifdef EAM_DATASTRINGS
     out->label = NULL;
-#endif
 }
 
 
-/* DJS (20 Aug 2004):  A more precise double version of map3d_xy() is
- * is required for the image routine.  The original intention was to
- * reuse the double version of the routine to generate the unsigned
- * int versin of the routine.  However, that caused rounding problems
- * such that PostScript versions of all the demos didn't come out
- * quite exactly the same.
- *
- * The define switch below will allow either code reuse or code
- * replication.  My advice is to study the rounding problem and
- * decide if the code reuse rounding is just as well as the code
- * replication approach.  If so, go the code reuse route and toss
- * the replicated code.
- */
-#define REPLICATE_CODE_FOR_BACKWARD_COMPATIBLE_ROUNDING 1
-
-#if REPLICATE_CODE_FOR_BACKWARD_COMPATIBLE_ROUNDING
-
 /* Function to map from user 3D space to normalized 'camera' view
  * space, and from there directly to terminal coordinates */
 void
 map3d_xy(
     double x, double y, double z,
-    unsigned int *xt, unsigned int *yt)
+    int *xt, int *yt)
 {
-    int i, j;
-    double v[4], res[4],	/* Homogeneous coords. vectors. */
-     w = trans_mat[3][3];
-
-    v[0] = map_x3d(x);		/* Normalize object space to -1..1 */
-    v[1] = map_y3d(y);
-    v[2] = map_z3d(z);
-    v[3] = 1.0;
-
-    for (i = 0; i < 2; i++) {	/* Dont use the third axes (z). */
-	res[i] = trans_mat[3][i];	/* Initiate it with the weight factor */
-	for (j = 0; j < 3; j++)
-	    res[i] += v[j] * trans_mat[j][i];
-    }
-
-    for (i = 0; i < 3; i++)
-	w += v[i] * trans_mat[i][3];
-    if (w == 0)
-	w = 1e-5;
-
-    if (lmargin.scalex == screen || rmargin.scalex == screen)
-	*xt = res[0] * xscaler/w + xmiddle;
-    else
-	*xt = (unsigned int) ((res[0] * xscaler / w) + xmiddle);
-
-    if (tmargin.scalex == screen || bmargin.scalex == screen)
-	*yt = res[1] * yscaler/w + ymiddle;
-    else
-	*yt = (unsigned int) ((res[1] * yscaler / w) + ymiddle);
-}
-
-/* Function to map from user 3D space to normalized 'camera' view
- * space, and from there directly to terminal coordinates */
-void
-map3d_xy_double(
-    double x, double y, double z,
-    double *xt, double *yt)
-{
-    int i, j;
-    double v[4], res[4],	/* Homogeneous coords. vectors. */
-     w = trans_mat[3][3];
-
-    v[0] = map_x3d(x);		/* Normalize object space to -1..1 */
-    v[1] = map_y3d(y);
-    v[2] = map_z3d(z);
-    v[3] = 1.0;
-
-    for (i = 0; i < 2; i++) {	/* Dont use the third axes (z). */
-	res[i] = trans_mat[3][i];	/* Initiate it with the weight factor */
-	for (j = 0; j < 3; j++)
-	    res[i] += v[j] * trans_mat[j][i];
-    }
-
-    for (i = 0; i < 3; i++)
-	w += v[i] * trans_mat[i][3];
-    if (w == 0)
-	w = 1e-5;
-
-    if (lmargin.scalex == screen || rmargin.scalex == screen)
-	*xt = res[0] * xscaler + xmiddle;
-    else
-	*xt = (res[0] * xscaler / w) + xmiddle;
-
-    if (tmargin.scalex == screen || bmargin.scalex == screen)
-	*yt = res[1] * yscaler + ymiddle;
-    else
-	*yt = (res[1] * yscaler / w) + ymiddle;
-}
-
-#else /* REPLICATE_CODE_FOR_BACKWARD_COMPATIBLE_ROUNDING */
-
-/* Function to map from user 3D space to normalized 'camera' view
- * space, and from there directly to terminal coordinates */
-void
-map3d_xy(
-    double x, double y, double z,
-    unsigned int *xt, unsigned int *yt)
-{
-#ifdef WITH_IMAGE
     double xtd, ytd;
     map3d_xy_double(x, y, z, &xtd, &ytd);
-    *xt = (unsigned int) xtd;
-    *yt = (unsigned int) ytd;
+    *xt = xtd;
+    *yt = ytd;
 }
 
 void
@@ -1020,38 +922,11 @@ map3d_xy_double(
     double x, double y, double z,
     double *xt, double *yt)
 {
-#endif
-    int i, j;
-    double v[4], res[4],	/* Homogeneous coords. vectors. */
-     w = trans_mat[3][3];
-
-    v[0] = map_x3d(x);		/* Normalize object space to -1..1 */
-    v[1] = map_y3d(y);
-    v[2] = map_z3d(z);
-    v[3] = 1.0;
-
-    for (i = 0; i < 2; i++) {	/* Dont use the third axes (z). */
-	res[i] = trans_mat[3][i];	/* Initiate it with the weight factor */
-	for (j = 0; j < 3; j++)
-	    res[i] += v[j] * trans_mat[j][i];
-    }
-
-    for (i = 0; i < 3; i++)
-	w += v[i] * trans_mat[i][3];
-    if (w == 0)
-	w = 1e-5;
-
-#ifdef WITH_IMAGE
-    *xt = ((res[0] * xscaler / w) + xmiddle);
-    *yt = ((res[1] * yscaler / w) + ymiddle);
-#else
-    *xt = (unsigned int) ((res[0] * xscaler / w) + xmiddle);
-    *yt = (unsigned int) ((res[1] * yscaler / w) + ymiddle);
-#endif
+    vertex v;
+    map3d_xyz(x, y, z, &v);
+    TERMCOORD(&v, *xt, *yt);
 }
 
-#endif /* REPLICATE_CODE_FOR_BACKWARD_COMPATIBLE_ROUNDING */
-
 
 /* HBB 20020313: New routine, broken out of draw3d_point, to be used
  * to output a single point without any checks for hidden3d */
@@ -1061,11 +936,9 @@ draw3d_point_unconditional(p_vertex v, struct lp_style_type *lp)
     unsigned int x, y;
 
     TERMCOORD(v, x, y);
+    /* Jul 2010 EAM - is it safe to overwrite like this? Make a copy instead? */
+    lp->pm3d_color.value = v->real_z;
     term_apply_lp_properties(lp);
-    /* HBB 20010822: implemented "linetype palette" for points, too */
-    if (lp->use_palette) {
-	set_color(cb2gray( z2cb(v->real_z) ));
-    }
     if (!clip_point(x, y))
 	(term->point) (x, y, lp->p_type);
 }
@@ -1077,7 +950,7 @@ void
 draw3d_line_unconditional(
     p_vertex v1, p_vertex v2,
     struct lp_style_type *lp,
-    int linetype)
+    t_colorspec color)
 {
     unsigned int x1, y1, x2, y2;
     struct lp_style_type ls = *lp;
@@ -1092,34 +965,35 @@ draw3d_line_unconditional(
     TERMCOORD(v1, x1, y1);
     TERMCOORD(v2, x2, y2);
 
-    /* User-specified line styles */
-    if (prefer_line_styles && linetype >= 0)
-	lp_use_properties(&ls, linetype+1, FALSE);
-
-    /* The usual case of auto-generated line types */
-    else
-	ls.l_type = linetype;
+    /* Replace original color with the one passed in */
+	ls.pm3d_color = color;
 
     /* Color by Z value */
     if (ls.pm3d_color.type == TC_Z)
 	    ls.pm3d_color.value = (v1->real_z + v2->real_z) * 0.5;
 
     term_apply_lp_properties(&ls);
-    draw_clip_line(x1,y1,x2,y2);
+
+    /* Support for hidden3d VECTOR mode with arrowheads */
+    if (lp->p_type == PT_ARROWHEAD)
+	draw_clip_arrow(x1,y1,x2,y2,END_HEAD);
+    else if (lp->p_type == PT_BACKARROW)
+	draw_clip_arrow(x1,y1,x2,y2,BACKHEAD);
+    else
+
+	draw_clip_line(x1,y1,x2,y2);
 }
 
 void
 draw3d_line (p_vertex v1, p_vertex v2, struct lp_style_type *lp)
 {
-#ifndef LITE
     /* hidden3d routine can't work if no surface was drawn at all */
     if (hidden3d && draw_surface) {
 	draw_line_hidden(v1, v2, lp);
 	return;
     }
-#endif
 
-    draw3d_line_unconditional(v1, v2, lp, lp->l_type);
+    draw3d_line_unconditional(v1, v2, lp, lp->pm3d_color);
 
 }
 
@@ -1128,14 +1002,12 @@ draw3d_line (p_vertex v1, p_vertex v2, struct lp_style_type *lp)
 void
 draw3d_point(p_vertex v, struct lp_style_type *lp)
 {
-#ifndef LITE
     /* hidden3d routine can't work if no surface was drawn at all */
     if (hidden3d && draw_surface) {
 	/* Draw vertex as a zero-length edge */
 	draw_line_hidden(v, NULL, lp);
 	return;
     }
-#endif
 
     draw3d_point_unconditional(v, lp);
 }
@@ -1152,23 +1024,22 @@ polyline3d_start(p_vertex v1)
     unsigned int x1, y1;
 
     polyline3d_previous_vertex = *v1;
-#ifndef LITE
     if (hidden3d && draw_surface)
 	return;
-#endif /* LITE */
 
+    /* EAM - This may now be unneeded. But I'm not sure. */
+    /*       Perhaps the hidden3d code needs the move.   */
     TERMCOORD(v1, x1, y1);
-    /* HBB FIXME 20031219: no clipping!? */
     term->move(x1, y1);
 }
 
 void
 polyline3d_next(p_vertex v2, struct lp_style_type *lp)
 {
+    unsigned int x1, y1;
     unsigned int x2, y2;
 
     /* Copied from draw3d_line(): */
-#ifndef LITE
     /* FIXME HBB 20031218: hidden3d mode will still create isolated
      * edges! */
     if (hidden3d && draw_surface) {
@@ -1176,22 +1047,31 @@ polyline3d_next(p_vertex v2, struct lp_style_type *lp)
 	polyline3d_previous_vertex = *v2;
 	return;
     }
-#endif
 
     /* Copied from draw3d_line_unconditional: */
     /* If use_palette is active, polylines can't be used -->
      * revert back to old method */
     if (lp->use_palette) {
 	draw3d_line_unconditional(&polyline3d_previous_vertex, v2,
-				  lp, lp->l_type);
+				  lp, lp->pm3d_color);
 	polyline3d_previous_vertex = *v2;
 	return;
 
     }
 
+    TERMCOORD(&polyline3d_previous_vertex, x1, y1);
     TERMCOORD(v2, x2, y2);
-    /* FIXME HBB 20031219: no clipping?! */
-    term->vector(x2, y2);
-
+    draw_clip_line(x1,y1,x2,y2);
     polyline3d_previous_vertex = *v2;
 }
+
+/*
+ * Dummy up an x-axis scale so that we can share the 2D arrowhead routine.
+ */
+void
+apply_3dhead_properties(struct arrow_style_type *arrow_properties)
+{
+    X_AXIS.term_scale = (plot_bounds.xright - plot_bounds.xleft)
+			/ (X_AXIS.max - X_AXIS.min);
+    apply_head_properties(arrow_properties);
+}
