@@ -1,5 +1,5 @@
 /*
- * $Id: winmain.c,v 1.52.2.5 2014/01/11 16:22:14 markisch Exp $
+ * $Id: winmain.c,v 1.52.2.8 2014/07/04 09:31:06 markisch Exp $
  */
 
 /* GNUPLOT - win/winmain.c */
@@ -441,7 +441,7 @@ int main(int argc, char **argv)
 
 
 #ifndef WGP_CONSOLE
-# if defined( __MINGW32__) && !defined(_W64)
+# if defined (__MINGW32__) && !defined(_W64)
 #  define argc _argc
 #  define argv _argv
 # else /* MSVC, WATCOM, MINGW-W64 */
@@ -557,7 +557,7 @@ int main(int argc, char **argv)
         if (TextInit(&textwin))
                 exit(1);
         textwin.hIcon = LoadIcon(hInstance, "TEXTICON");
-        SetClassLong(textwin.hWndParent, GCL_HICON, (DWORD)textwin.hIcon);
+        SetClassLongPtr(textwin.hWndParent, GCLP_HICON, (LONG_PTR)textwin.hIcon);
 	/* Note: we want to know whether this is an interactive session so that we can
 	 * decide whether or not to write status information to stderr.  The old test
 	 * for this was to see if (argc > 1) but the addition of optional command line
@@ -608,7 +608,7 @@ int main(int argc, char **argv)
 	gnu_main(argc, argv);
 
 	/* First chance to close help system for console gnuplot,
-	second for wgnuplot */
+	   second for wgnuplot */
 	WinCloseHelp();
 	return 0;
 }
@@ -846,6 +846,7 @@ MyFRead(void *ptr, size_t size, size_t n, FILE *file)
 
 #else /* WGP_CONSOLE */
 
+
 DWORD WINAPI stdin_pipe_reader(LPVOID param)
 {
 #if 0
@@ -863,6 +864,7 @@ DWORD WINAPI stdin_pipe_reader(LPVOID param)
 #endif
 }
 
+
 int ConsoleGetch()
 {
     int fd = fileno(stdin);
@@ -1020,7 +1022,7 @@ win_lower_terminal_group(void)
 
 
 /* returns true if there are any graph windows open (win terminal) */
-TBOOLEAN
+static TBOOLEAN
 WinWindowOpened(void)
 {
 	LPGW lpgw;
@@ -1035,15 +1037,23 @@ WinWindowOpened(void)
 }
 
 
-#ifndef WGP_CONSOLE
-void
-WinPersistTextClose(void)
+/* returns true if there are any graph windows open (wxt/win terminals) */
+TBOOLEAN
+WinAnyWindowOpen(void)
 {
 	TBOOLEAN window_opened = WinWindowOpened();
 #ifdef WXWIDGETS
 	window_opened |= wxt_window_opened();
 #endif
-	if (!window_opened &&
+	return window_opened;
+}
+
+
+#ifndef WGP_CONSOLE
+void
+WinPersistTextClose(void)
+{
+	if (!WinAnyWindowOpen() &&
 		(textwin.hWndParent != NULL) && !IsWindowVisible(textwin.hWndParent))
 		PostMessage(textwin.hWndParent, WM_CLOSE, 0, 0);
 }
