@@ -1,5 +1,5 @@
 /*
- *$Id: wtext.c,v 1.37.2.5 2014/01/11 16:22:14 markisch Exp $
+ *$Id: wtext.c,v 1.37.2.6 2014/07/04 07:45:44 markisch Exp $
  */
 
 /* GNUPLOT - win/wtext.c */
@@ -85,7 +85,7 @@
 /* limits */
 static POINT ScreenMinSize = {16,4};
 
-BOOL CALLBACK AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
+INT_PTR CALLBACK AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam);
 static LRESULT CALLBACK WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
 static LRESULT CALLBACK WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
 
@@ -287,7 +287,7 @@ TextInit(LPTW lptw)
 
     sysmenu = GetSystemMenu(lptw->hWndParent,0);	/* get the sysmenu */
     AppendMenu(sysmenu, MF_SEPARATOR, 0, NULL);
-    AppendMenu(sysmenu, MF_POPUP, (UINT)lptw->hPopMenu, "&Options");
+    AppendMenu(sysmenu, MF_POPUP, (UINT_PTR)lptw->hPopMenu, "&Options");
     AppendMenu(sysmenu, MF_STRING, M_ABOUT, "&About");
 
     if (lptw->lpmw)
@@ -962,6 +962,7 @@ TextMakeFont(LPTW lptw)
     return;
 }
 
+
 static void
 TextSelectFont(LPTW lptw) {
     LOGFONT lf;
@@ -1020,7 +1021,7 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     HDC hdc;
     LPTW lptw;
 
-    lptw = (LPTW)GetWindowLong(hwnd, 0);
+    lptw = (LPTW)GetWindowLongPtr(hwnd, 0);
 
     switch(message) {
     case WM_SYSCOMMAND:
@@ -1098,7 +1099,7 @@ WndParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 	TEXTMETRIC tm;
 
 	lptw = ((CREATESTRUCT *)lParam)->lpCreateParams;
-	SetWindowLong(hwnd, 0, (LONG)lptw);
+	SetWindowLongPtr(hwnd, 0, (LONG_PTR)lptw);
 	lptw->hWndParent = hwnd;
 	/* get character size */
 	TextMakeFont(lptw);
@@ -1195,7 +1196,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     int nYinc, nXinc;
     LPTW lptw;
 
-    lptw = (LPTW)GetWindowLong(hwnd, 0);
+    lptw = (LPTW)GetWindowLongPtr(hwnd, 0);
 
     switch(message) {
     case WM_SETFOCUS:
@@ -1870,7 +1871,7 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
     }
     case WM_CREATE:
 	lptw = ((CREATESTRUCT *)lParam)->lpCreateParams;
-	SetWindowLong(hwnd, 0, (LONG)lptw);
+	SetWindowLongPtr(hwnd, 0, (LONG_PTR)lptw);
 	lptw->hWndText = hwnd;
 	break;
     case WM_DESTROY:
@@ -1891,6 +1892,7 @@ TextKBHit(LPTW lptw)
     return (lptw->KeyBufIn != lptw->KeyBufOut);
 }
 
+
 /* get character from keyboard, no echo */
 /* need to add extended codes */
 int WDPROC
@@ -1932,6 +1934,7 @@ TextGetChE(LPTW lptw)
     return ch;
 }
 
+
 LPSTR WDPROC
 TextGetS(LPTW lptw, LPSTR str, unsigned int size)
 {
@@ -1960,6 +1963,7 @@ TextGetS(LPTW lptw, LPSTR str, unsigned int size)
     return str;
 }
 
+
 int WDPROC
 TextPutS(LPTW lptw, LPSTR str)
 {
@@ -2139,7 +2143,7 @@ ReadTextIni(LPTW lptw)
 
 
 /* About Box */
-BOOL CALLBACK
+INT_PTR CALLBACK
 AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
 {
     switch (wMsg) {
@@ -2155,7 +2159,7 @@ AboutDlgProc(HWND hDlg, UINT wMsg, WPARAM wParam, LPARAM lParam)
     case WM_DRAWITEM:
     {
 	LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT)lParam;
-	DrawIcon(lpdis->hDC, 0, 0, (HICON)GetClassLong(GetParent(hDlg), GCL_HICON));
+	DrawIcon(lpdis->hDC, 0, 0, (HICON)GetClassLongPtr(GetParent(hDlg), GCLP_HICON));
 	return FALSE;
     }
     case WM_COMMAND:
